using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.Threading;
using System.Net;
using System.IO;
using System.Collections;
using System.Text.RegularExpressions;
using System.Diagnostics;

namespace EbiSoft.ZeroProxy.Proxy
{
    public class ProxyConnection : IDisposable
    {
        private readonly ProxyServer owner;
        private readonly Socket      socket;
        private Thread therad;

        private const int BufferSize = 1024;
        private static readonly Regex requestRegex = new Regex(@"^([A-Z]+)\s(http|https)://([^/\s]+)([^\s]*)\sHTTP/(\d.\d)", RegexOptions.Compiled);

        /// <summary>
        /// RXgN^
        /// </summary>
        /// <param name="owner">T[o[</param>
        /// <param name="socket">\Pbg</param>
        public ProxyConnection(ProxyServer owner, Socket socket)
        {
            this.owner = owner;
            this.socket = socket;
            this.therad = new Thread(new ThreadStart(TheadAction));

            UpdateTick();
            m_active = true;
            therad.IsBackground = true;
            therad.Start();
        }

        /// <summary>
        /// Xbh
        /// </summary>
        private void TheadAction()
        {
            try
            {
                ProxyAction();
            }
            catch (ThreadAbortException) { }
            finally
            {
                m_active = false;
                therad = null;
            }
        }

        /// <summary>
        /// vNV
        /// </summary>
        private void ProxyAction()
        {
            ArrayList processBuffer = new ArrayList();
            string    header;
            Socket    originSocket = null;
            try
            {
                using (NetworkStream clientStream = new NetworkStream(socket))
                {
                    Debug.WriteLine("Phase.1");

                    // wb_M
                    processBuffer.Clear();
                    header = receiveHeader(clientStream, processBuffer);

                    // NGXgwb_擾
                    Match match = requestRegex.Match(header);
                    if (!match.Success)
                    {
                        // NGXgB
                        sendErrorPage(clientStream, "400", "Bad Request");
                        return;
                    }
                    string method   = match.Groups[1].Value.ToUpper();
                    string protocol = match.Groups[2].Value;
                    string host     = match.Groups[3].Value;
                    string reqPath  = match.Groups[4].Value;
                    string httpVer  = match.Groups[5].Value;

                    // \bhGETPOST̂݃T|[g
                    if (method != "GET" && method != "POST")
                    {
                        // \bh
                        sendErrorPage(clientStream, "501", "Not Implemented");
                        return;
                    }
                    
                    // NGXg̍ŏ̍s܂܂鍷ւ
                    int reqLineLength = Encoding.ASCII.GetByteCount(match.Groups[0].Value);
                    processBuffer.RemoveRange(0, reqLineLength);
                    processBuffer.InsertRange(0, Encoding.ASCII.GetBytes(method + " " + reqPath + " HTTP/1.0"));

                    // ܂f[^cĂȂSM
                    if (socket.Poll(1000, SelectMode.SelectRead))
                    {
                        while (receiveFromStream(clientStream, processBuffer).Length > 0) { }
                    }

                    // |[gԍ擾
                    int port = 80;      // ftHg:80
                    if (host.IndexOf(':')>-1)
                    {
                        // |[gԍw肪ꍇ
                        try
                        {
                            port = int.Parse(host.Substring(host.IndexOf(':') + 1));
                            host = host.Substring(0, host.IndexOf(':'));
                        }
                        catch (Exception)
                        {
                            sendErrorPage(clientStream, "400", "Bad Request");
                        }
                    }
                    else if (protocol.ToLower() == "https")
                    {
                        port = 443;
                    }

                    Debug.WriteLine("Phase.2");

                    // Ώۂ̃T[o[֐ڑ
                    UpdateTick();
                    IPAddress addr = Dns.GetHostEntry(host).AddressList[0];
                    originSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                    originSocket.Connect(new IPEndPoint(addr, port));
                    using (NetworkStream originStream = new NetworkStream(originSocket))
                    {
                        // NGXg܂Ƃ߂đM
                        byte[] sendBuffer = new byte[processBuffer.Count];
                        processBuffer.CopyTo(sendBuffer);
                        originStream.Write(sendBuffer, 0, sendBuffer.Length);

                        // wb_M
                        processBuffer.Clear();
                        header = receiveHeader(originStream, processBuffer);

                        // wb_
                        processBuffer.RemoveRange(0, Encoding.ASCII.GetByteCount(header));

                        // ǉwb_t
                        header += "\nProxy-connection: close";

                        Debug.WriteLine("Phase.3");

                        // wb_M
                        sendBuffer = Encoding.ASCII.GetBytes(header);
                        clientStream.Write(sendBuffer, 0, sendBuffer.Length);

                        // processBuffer cf[^M
                        if (processBuffer.Count > 0)
                        {
                            sendBuffer = new byte[processBuffer.Count];
                            processBuffer.CopyTo(sendBuffer);
                            clientStream.Write(sendBuffer, 0, sendBuffer.Length);
                        }
                        Debug.WriteLine("Phase.4");

                        // ܂f[^cĂȂSM
                        while (true)
                        {
                            sendBuffer = receiveFromStream(originStream, null);
                            if (sendBuffer.Length == 0)
                            {
                                break;
                            }
                            clientStream.Write(sendBuffer, 0, sendBuffer.Length);
                        }

                        Debug.WriteLine("Phase.5");
                    }
                }
            }
            catch (IOException) { }
            catch (SocketException) { }
            finally
            {
                if ((originSocket != null) && (originSocket.Connected))
                {
                    originSocket.Close();
                }
                if (socket.Connected)
                {
                    socket.Close();
                }
            }
        }

        /// <summary>
        /// wb_M܂
        /// </summary>
        /// <param name="readStream">MXg[</param>
        /// <param name="processBuffer">Mf[^i[ArrayList</param>
        /// <returns>Mwb_</returns>
        private string receiveHeader(NetworkStream readStream, ArrayList processBuffer)
        {
            byte[] receiveBuffer;
            string receivedStr   = string.Empty;
            StringBuilder sb     = new StringBuilder();

            // M[v
            while (true)
            {
                // Xg[ǂݎAɂ
                receiveBuffer = receiveFromStream(readStream, processBuffer);
                receivedStr = Encoding.ASCII.GetString(receiveBuffer, 0, receiveBuffer.Length);

                // I[`FbN
                if (receivedStr.IndexOf("\r\n\r\n") > -1)
                {
                    // I[܂łǉ
                    sb.Append(receivedStr.Substring(0, receivedStr.IndexOf("\r\n\r\n")));
                    break;
                }
                else
                {
                    // ׂĒǉ
                    sb.Append(receivedStr);
                }
            }

            // wb_Ԃ
            return sb.ToString();
        }

        /// <summary>
        /// Xg[f[^M܂
        /// </summary>
        /// <param name="readStream">ǂݍ݂Ƃ̃Xg[</param>
        /// <param name="processBuffer">Mf[^i[Ăobt@</param>
        /// <returns>Mf[^</returns>
        private byte[] receiveFromStream(NetworkStream readStream, ArrayList processBuffer)
        {
            // obt@1M
            byte[] buffer = new byte[BufferSize];
            int receivedBytes = readStream.Read(buffer, 0, buffer.Length);
            UpdateTick();

            if (receivedBytes == 0)
            {
                // MoCgȂB
                return new byte[] { };
            }
            else if (receivedBytes < buffer.Length)
            {
                // obt@^܂œǂݎȂꍇ́AꂽƂ܂łԂ
                byte[] copyTemp = new byte[receivedBytes];
                Array.Copy(buffer, 0, copyTemp, 0, receivedBytes);
                if (processBuffer != null)
                {
                    processBuffer.AddRange(copyTemp);
                }
                return copyTemp;
            }
            else
            {
                // Mς݃f[^ׂĂԂ
                if (processBuffer != null)
                {
                    processBuffer.AddRange(buffer);
                }
                return buffer;
            }
        }

        /// <summary>
        /// G[y[W𑗐M܂
        /// </summary>
        /// <param name="stream">MXg[</param>
        /// <param name="errorNumber">G[ԍ</param>
        /// <param name="errorReason">G[R</param>
        private void sendErrorPage(NetworkStream stream, string errorNumber, string errorReason)
        {
            byte[] sendBuffer;
            StringBuilder sb = new StringBuilder();
            StringWriter writer = new StringWriter(sb);
            writer.NewLine = "\r\n";

            // X|Xwb_M
            writer.WriteLine("HTTP/1.0 {0} {1}", errorNumber, errorReason);
            writer.WriteLine("Content-Type: text/html");
            writer.WriteLine("Proxy-connection: close");
            writer.WriteLine();
            sendBuffer = Encoding.ASCII.GetBytes(sb.ToString());
            stream.Write(sendBuffer, 0, sendBuffer.Length);

            // NA
            sb = new StringBuilder();
            writer = new StringWriter(sb);

            // X|X{fB𑗐M
            writer.WriteLine("<html>");
            writer.WriteLine("<head>");
            writer.WriteLine("<title>{0} {1}</title>");
            writer.WriteLine("</head>");
            writer.WriteLine("<body>");
            writer.WriteLine("<h1>{0} {1}</h1>");
            writer.WriteLine("</body>");
            writer.WriteLine("<html>");
            writer.WriteLine();
            sendBuffer = Encoding.ASCII.GetBytes(sb.ToString());
            stream.Write(sendBuffer, 0, sendBuffer.Length);
        }

        /// <summary>
        /// ŏIʐMԂXV܂B
        /// </summary>
        private void UpdateTick()
        {
            m_lastCommunicated = Environment.TickCount;
        }

        #region IDisposable o

        public void Dispose()
        {
            if (therad != null)
                therad.Abort();
        }

        #endregion

        #region vpeB

        /// <summary>
        /// ANeBuԂǂׂ܂
        /// </summary>
        public bool Active
        {
            get { return m_active; }
            set { m_active = value; }
        }
        private bool m_active = true;

        /// <summary>
        /// Ō̒ʐMTick𒲂ׂ܂
        /// </summary>
        public int LastCommunicated
        {
            get { return m_lastCommunicated; }
        }
        private int m_lastCommunicated = 0;

        #endregion
    }
}
