using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Net;

namespace EbiSoft.ZeroProxy
{
    public class IpHelperWraper
    {
        private class IpHelperMethods
        {
            #region P/Invoke 錾
#if PocketPC
            // Methods
            [DllImport("coredll", EntryPoint = "memcpy")]
            public static extern void CopyMemoryFixedInfo([Out] FixedInfo dest, byte[] source, int size);
            [DllImport("coredll", EntryPoint = "memcpy")]
            public static extern void CopyMemoryInt(ref int dest, ref byte source, int size);
            [DllImport("coredll", EntryPoint = "memcpy")]
            public static extern void CopyMemoryIPAdapterIndexMap(ref IPAdapterIndexMap dest, ref byte source, int size);
            [DllImport("coredll", EntryPoint = "memcpy")]
            public static extern void CopyMemoryIPAdapterInfo(ref IPAdapterInfo dest, byte[] source, int size);
            [DllImport("coredll", EntryPoint = "memcpy")]
            public static extern void CopyMemoryIPAdapterInfoP(ref IPAdapterInfo dest, IntPtr source, int size);
            [DllImport("coredll", EntryPoint = "memcpy")]
            public static extern void CopyMemoryIPAddrString(ref IPAddrString dest, IntPtr source, int size);
            [DllImport("Iphlpapi.dll")]
            public static extern int GetAdaptersInfo(byte[] PAdapterInfoBuffer, ref int size);
            [DllImport("Iphlpapi.dll")]
            public static extern int GetInterfaceInfo(byte[] PIfTableBuffer, ref int size);
            [DllImport("Iphlpapi.dll")]
            public static extern int GetNetworkParams(byte[] PFixedInfoBuffer, ref int size);
            [DllImport("Iphlpapi.dll")]
            public static extern int IpReleaseAddress(ref IPAdapterIndexMap AdapterInfo);
            [DllImport("Iphlpapi.dll")]
            public static extern int IpRenewAddress(ref IPAdapterIndexMap AdapterInfo);
#else
            [DllImport("Kernel32.dll", EntryPoint = "CopyMemory", CharSet = CharSet.Auto, SetLastError = true)]
            public static extern void CopyMemoryFixedInfo([Out] FixedInfo dest, byte[] source, int size);
            [DllImport("Kernel32.dll", EntryPoint = "CopyMemory", CharSet = CharSet.Auto, SetLastError = true)]
            public static extern void CopyMemoryInt(ref int dest, ref byte source, int size);
            [DllImport("Kernel32.dll", EntryPoint = "CopyMemory", CharSet = CharSet.Auto, SetLastError = true)]
            public static extern void CopyMemoryIPAdapterIndexMap(ref IPAdapterIndexMap dest, ref byte source, int size);
            [DllImport("Kernel32.dll", EntryPoint = "CopyMemory", CharSet = CharSet.Auto, SetLastError = true)]
            public static extern void CopyMemoryIPAdapterInfo(ref IPAdapterInfo dest, byte[] source, int size);
            [DllImport("Kernel32.dll", EntryPoint = "CopyMemory", CharSet = CharSet.Auto, SetLastError = true)]
            public static extern void CopyMemoryIPAdapterInfoP(ref IPAdapterInfo dest, IntPtr source, int size);
            [DllImport("Kernel32.dll", EntryPoint = "CopyMemory", CharSet = CharSet.Auto, SetLastError = true)]
            public static extern void CopyMemoryIPAddrString(ref IPAddrString dest, IntPtr source, int size);
            [DllImport("Iphlpapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
            public static extern int GetAdaptersInfo(byte[] PAdapterInfoBuffer, ref int size);
            [DllImport("Iphlpapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
            public static extern int GetInterfaceInfo(byte[] PIfTableBuffer, ref int size);
            [DllImport("Iphlpapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
            public static extern int GetNetworkParams(byte[] PFixedInfoBuffer, ref int size);
            [DllImport("Iphlpapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
            public static extern int IpReleaseAddress(ref IPAdapterIndexMap AdapterInfo);
            [DllImport("Iphlpapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
            public static extern int IpRenewAddress(ref IPAdapterIndexMap AdapterInfo);
#endif
            #endregion
        }

        public static IPAdapterInfo[] GetAdaptersInfo()
        {
            int size = 0;
            int retValue = IpHelperMethods.GetAdaptersInfo(null, ref size);
            if ((retValue != 0) & (retValue != 0x6f))
            {
                throw new Exception("Error invoking GetAdaptersInfo()");
            }

            List<IPAdapterInfo> infoList = new List<IPAdapterInfo>();
            
            byte[] IPAdapterInfoBuffer = new byte[size + 1];
            IPAdapterInfo PAdapterInfo = new IPAdapterInfo();
            retValue = IpHelperMethods.GetAdaptersInfo(IPAdapterInfoBuffer, ref size);
            if (retValue != 0)
            {
                throw new Exception("Error invoking GetAdaptersInfo()");
            }
            IpHelperMethods.CopyMemoryIPAdapterInfo(ref PAdapterInfo, IPAdapterInfoBuffer, Marshal.SizeOf(PAdapterInfo));

            IntPtr AdapterInfoNext = new IntPtr();
            do
            {
                infoList.Add(PAdapterInfo);
                AdapterInfoNext = PAdapterInfo.NextPointer;
                if (AdapterInfoNext != IntPtr.Zero)
                {
                    IpHelperMethods.CopyMemoryIPAdapterInfoP(ref PAdapterInfo, AdapterInfoNext, Marshal.SizeOf(PAdapterInfo));
                }
            } while (AdapterInfoNext != IntPtr.Zero);

            return infoList.ToArray();
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public class FixedInfo
    {
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 0x84)]
        public string HostName;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 0x84)]
        public string DomainName;
        public IntPtr CurrentServerList;
        public IPAddrString DnsServerList;
        public int NodeType;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
        public string ScopeId;
        public int EnableRouting;
        public int EnableProxy;
        public int EnableDns;
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
    public struct IPAdapterIndexMap
    {
        public int Index;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 0x80)]
        public string Name;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct IPAdapterInfo
    {
        public IntPtr NextPointer;
        public int ComboIndex;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 260)]
        public byte[] AdapterName;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 0x84)]
        public byte[] Description;
        public int AddressLength;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public byte[] Address;
        public int Index;
        public int Type;
        public int DhcpEnabled;
        public IntPtr CurrentIPAddress;
        public IPAddrString IPAddressList;
        public IPAddrString GatewayList;
        public IPAddrString DhcpServer;
        public bool HaveWins;
        public IPAddrString PrimaryWinsServer;
        public IPAddrString SecondaryWinsServer;
        public int LeaseObtained;
        public int LeaseExpires;

        public string GetAdapterName()
        {
            return Encoding.Default.GetString(AdapterName,0, AdapterName.Length).TrimEnd('\0');
        }
    }

    public class IPAdapterTypes
    {
        // Fields
        public const int MIB_IF_TYPE_ETHERNET = 6;
        public const int MIB_IF_TYPE_FDDI = 15;
        public const int MIB_IF_TYPE_LOOPBACK = 0x18;
        public const int MIB_IF_TYPE_OTHER = 1;
        public const int MIB_IF_TYPE_PPP = 0x17;
        public const int MIB_IF_TYPE_SLIP = 0x1c;
        public const int MIB_IF_TYPE_TOKENRING = 9;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct IPAddrString
    {
        public IntPtr NextPointer;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 0x10)]
        public byte[] IPAddressString;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 0x10)]
        public byte[] IPMaskString;
        public int Context;

        public string GetIPAddressString()
        {
            return Encoding.Default.GetString(IPAddressString, 0, IPAddressString.Length).TrimEnd('\0');
        }

        public string GetIPMaskString()
        {
            return Encoding.Default.GetString(IPMaskString, 0, IPMaskString.Length).TrimEnd('\0');
        }
    }

    public class IPConfigConst
    {
        public const int DEFAULT_MINIMUM_ENTITIES = 0x20;
        public const int ERROR_BUFFER_OVERFLOW = 0x6f;
        public const int ERROR_SUCCESS = 0;
        public const int MAX_ADAPTER_ADDRESS_LENGTH = 8;
        public const int MAX_ADAPTER_DESCRIPTION_LENGTH = 0x80;
        public const int MAX_ADAPTER_NAME = 0x80;
        public const int MAX_ADAPTER_NAME_LENGTH = 0x100;
        public const int MAX_DOMAIN_NAME_LEN = 0x80;
        public const int MAX_HOSTNAME_LEN = 0x80;
        public const int MAX_SCOPE_ID_LEN = 0x100;
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
    public struct IPInterfaceInfo
    {
        public int NumAdapters;
        public IPAdapterIndexMap Adapter;
    }
}
